import numpy as np

def channelshuffle(img) -> np.ndarray:
    """
    Method randomly shuffles image's channels (swaps them places).

    :return: np.ndarray
    """
    img_cp = img.copy()

    img_cp = img_cp[:, :, np.random.permutation(3)]

    return img_cp


def colorjitter(img, channel, a, add_brightness = 0) -> np.ndarray:
    """
    Method increases intensity of the selected channel by parameter alpha and optionally
    increases/decreases its brightness.

    :param channel: int(0 <= channel < 3) - selected channel to jitter (0 = R, 1 = G, 2 = B).
    :param a: float(-1 <= alpha <= 1) - the amount of intensity to apply.
    :param add_brightness: float(-1 <= add_brightness <= 1) - the amount of brightness to add (default = 0).
    :return: np.ndarray
    """
    img_cp = img.copy()

    alpha = np.floor(a * 255)
    if alpha > 0:
        img_cp[:, :, channel] = np.where((255 - img_cp[:, :, channel]) < alpha, 255,
                                             img_cp[:, :, channel] + alpha)
    if alpha < 0:
        img_cp[:, :, channel] = np.where((255 + img_cp[:, :, channel] + alpha) < 0, 0,
                                             img_cp[:, :, channel] + alpha)

    alpha_b = np.floor(add_brightness * 255)
    if alpha_b > 0:
        img_cp = np.where((255 - img_cp) < alpha_b, 255, img_cp + alpha_b)
    if alpha_b < 0:
        img_cp = np.where((255 + img_cp + alpha) < 0, 0, img_cp + alpha_b)
    img_cp = np.array(img_cp, dtype = np.uint8)

    return img_cp


import numpy as np
from imgic_resize import resize

def combine(img, target_image, a=0.5) -> np.ndarray:
    """
    Method combines the original image with a target image (rescaled to the same dims),
    using an opacity parameter a.

    :param target_image: Img() - an Img class instance type image, to combine with the first image.
    :param a: float(0 <= a <= 1) - opacity parameter (1 = original image only, 0 = target image only)
    :return: np.ndarray
    """
    img_cp = img.copy()

    if img_cp.shape != target_image.shape:
        target_image = resize(target_image, height = img_cp.shape[0], width = img_cp.shape[1])

    img_cp = np.array(target_image * (1 - a), dtype = np.uint16) + \
                 np.array(img_cp * a, dtype = np.uint16)
    return img_cp



import numpy as np

def blur(img, a = 1) -> np.ndarray:
        """
        Method applies Gaussian blurring to the input, with the strength alpha. The method
        maintains the original size of the object, but inserts a black border.

        :param a: int(1 <= a < inf) - parameter to repeat the blurring a times. Strengthens the effect.
        :return: np.ndarray
        """

        kernel = np.array([[0.0625, 0.125, 0.00625], [0.125, 0.25, 0.125], [0.0625, 0.125, 0.0625]])
        img_cp = img.copy()
        backup = img_cp.copy()

        for repeat in range(a):
            for channel in range(3):
                for row in range(0, img_cp.shape[0] - 2):
                    for col in range(0, img_cp.shape[1] - 2):
                        img_cp[row + 1, col + 1, channel] = np.floor(np.sum(backup[row:(row + 3),
                                                                                col:(col + 3), channel] * kernel))
        img_cp[[0, -1], :, :] = 0
        img_cp[: ,[0, -1], :] = 0

        return img_cp



import numpy as np

def resize(img, a=0, height=0, width=0) -> np.ndarray:
    """
    Method resizes image to either fixed height/width or to size equal to
    proportion of original size a.

    :param a: float(0 < a) - proportional resizing parameter alpha (returned dims = a * original dims)
    :param height: int(0 < height) - height of resized image (in px)
    :param width: int(0 < width) - width of resized image (in px)
    :return: np.ndarray
    """

    img_cp = img.copy()

    rowlen = np.round(img_cp.shape[0] * (a)) if (height == 0) & (width == 0) else height
    collen = np.round(img_cp.shape[1] * (a)) if (height == 0) & (width == 0) else width

    img_cp = img_cp[np.linspace(0, img_cp.shape[0] - 1, rowlen, dtype = np.uint16), :, :]
    img_cp = img_cp[:, np.linspace(0, img_cp.shape[1] - 1, collen, dtype = np.uint16), :]

    return img_cp


import numpy as np

def dropout(img, a = 0.5) -> np.ndarray:
    """
    Method drops out the selected fraction of pixels from the input image (replaces
    with black pixels).

    :param a: float(0 <= a <= 1) - the fraction of pixels to remove from the image (default = 0.5 = 50%)
    :return: np.ndarray
    """
    img_cp = img.copy()

    img_cp[np.random.choice([0, 1], size = img_cp.shape[0:2],
                                p = [1 - a, a]).astype(np.bool)] = [0, 0, 0]

    return img_cp


import numpy as np

def flip(img, axis = 0) -> np.ndarray:
    """
    Method flips the image around either vertically (over horizontal axis = 0),
    or horizontally (over vertical axis = 1).

    :param axis: int(0/1). Flip the image vertically (over horizontal axis = 0) or vertically (over vertical axis = 1)
    :return: np.ndarray
    """
    img_cp = img.copy()

    if axis == 0:
        img_cp = np.flip(img_cp, axis = 0)
    if axis == 1:
        img_cp = np.flip(img_cp, axis = 1)

    return img_cp



from setuptools import setup, find_namespace_packages

setup(
    name = "imgic",
    version = "0.2.1",
    description = "A basic numpy-based image manipulation package. Contains tools for resizing, cropping, blurring, and others.",
    url = "https://github.com/LukasNorbutas/imgic",
    author = "Lukas Norbutas",
    author_email = "lukas.norbutas@gmail.com",
    classifiers = ['Development Status :: 3 - Alpha',
                   'Programming Language :: Python :: 3.5',
                   'Programming Language :: Python :: 3.6',
                   'Programming Language :: Python :: 3.7'],
    packages = find_namespace_packages(),
    python_requires = ">=3.5, <4",
    install_requires = ['numpy==1.16.3']
)



import numpy as np
from imgic_resize import resize


def crop(img, px=[0, 0, 0, 0], pc=[0, 0, 0, 0], tp_l = [], bt_r = [], keep_size = 0) -> np.ndarray:
    """
    This method crops the image, based either on an input list of the amount of pixels
    to crop (top/right/bottom/left), percentage of pixels to crop, or two coordinates:
    top_left, and bottom_right corner.

    :param px: list(top, right, bottom, left) - number of pixels to crop at each side.
    :param pc: list(top, right, bottom, left) - percentage of pixels to crop at each side.
    :param tp_l: list(row, column) - coordinate of top-left corner for coordinate based cropping.
    :param bt_r: list(row, column) - coordinate of bottom-right corner for coordinate based cropping.
    :param keep_size: int(1/0) - resize cropped image to original size (default = 0).
    :return: np.ndarray
    """
    img_cp = img.copy()

    if tp_l != []:
        px[0], px[3] = tp_l[0], tp_l[1]
    if bt_r != []:
        px[2], px[1] = bt_r[0], bt_r[1]
    if pc != [0, 0, 0, 0]:
        while len(pc) < 4:
            pc.append(0)
        px[0], px[1], px[2], px[3] = round(img_cp.shape[0] * pc[0]), round(img_cp.shape[0] * pc[1]), \
                                     round(img_cp.shape[1] * pc[2]), round(img_cp.shape[1] * pc[3])
    if px != [0, 0, 0, 0]:
        while len(px) < 4:
            px.append(0)
    img_cp = img_cp[0 + px[0]:img_cp.shape[0] - px[2],
                 0 + px[3]:img_cp.shape[1] - px[1], 0:3]

    if keep_size == 1:
        img_cp = resize(img_cp, height = img_cp.shape[0], width = img_cp.shape[1])

    return img_cp


def fixed_crop(img, height = 150, width = 150, v_pos = "center", h_pos = "center", keep_size = 0) -> np.ndarray:
    """
    Method crops the image to selected height/width, at the specific
    location of the image along horizontal/vertical axes (center, top/bottom-left/right, random).

    :param height: int() - crop to height (in px)
    :param width:  int() - crop to width (in px)
    :param v_pos: str("center"/"top"/"bottom"/"random") - crops image at selected vertical location
    :param h_pos: str("center"/"left"/"right"/"bottom") - crops image at selected horizontal location
    :param keep_size: keep_size: int(1/0) - resize cropped image to original size (default = 0)
    :return: np.ndarray
    """
    img_cp = img.copy()

    if v_pos == "center":
        margins = np.array([np.floor((img_cp.shape[0] - height) / 2),
                            np.floor((img_cp.shape[0] - (img_cp.shape[0] - height) / 2))],
                           dtype=np.uint16)
    if v_pos == "top":
        margins = np.array([0, height], dtype = np.uint16)
    if v_pos == "bottom":
        margins = np.array([img_cp.shape[0] - height, img_cp.image.shape[0]], dtype = np.uint16)
    if v_pos == "random":
        rand_gen = np.random.randint(0, img_cp.shape[0] - height)
        margins = np.array([rand_gen, rand_gen + width], dtype = np.uint16)

    if h_pos == "center":
        margins = np.append(margins, np.array([np.floor((img_cp.shape[1] - width) / 2),
                                               img_cp.shape[1] - np.ceil((img_cp.shape[1] - width) / 2)],
                                              dtype = np.uint16))
    if h_pos == "left":
        margins = np.append(margins, np.array([0, width], dtype = np.uint16))
    if h_pos == "right":
        margins = np.append(margins, np.array([img_cp.shape[1] - width, img_cp.shape[1]], dtype = np.uint16))
    if h_pos == "random":
        rand_gen = np.random.randint(0, img_cp.shape[1] - width)
        margins = np.append(margins, np.array([rand_gen, rand_gen + width], dtype = np.uint16))

    img_cp = img_cp[margins[0]:margins[1], margins[2]:margins[3], 0:3]

    if keep_size == 1:
        img_cp = resize(img_cp, height = img_cp.shape[0], width = img_cp.shape[1])

    return img_cp



import numpy as np
import imgic_resize as resize
import imgic_blur as blur
import imgic_color_trans as color_trans
import imgic_crop as crop
import imgic_dropout as dropout
import imgic_flip as flip
import imgic_combine as combine


class Img:
    def __init__(self, image: np.ndarray):
        self.image = image
        self.shape = self.image.shape

    def __repr__(self):
        return f'Image object of {self.shape[0]}x{self.shape[1]}'

    def crop(self, px = [0, 0, 0, 0], pc = [0, 0, 0, 0], tp_l = [], bt_r = [], keep_size = 0) -> 'Img':
        """
        This method crops the image, based either on an input list of the amount of pixels
        to crop (top/right/bottom/left), percentage of pixels to crop, or two coordinates:
        top_left, and bottom_right corner.

        :param px: list(top, right, bottom, left) - number of pixels to crop at each side.
        :param pc: list(top, right, bottom, left) - percentage of pixels to crop at each side.
        :param tp_l: list(row, column) - coordinate of top-left corner for coordinate based cropping.
        :param bt_r: list(row, column) - coordinate of bottom-right corner for coordinate based cropping.
        :param keep_size: int(1/0) - resize cropped image to original size (default = 0).
        :return: Img class instance
        """
        self.image = crop.crop(self.image, px, pc, tp_l, bt_r, keep_size)
        self.shape = self.image.shape
        return self

    def fixed_crop(self, height = 150, width = 150, v_pos = "center", h_pos = "center", keep_size=0) -> 'Img':
        """
        Method crops the image to selected height/width, at the specific
        location of the image along horizontal/vertical axes (center, top/bottom-left/right, random).

        :param height: int() - crop to height (in px)
        :param width:  int() - crop to width (in px)
        :param v_pos: str("center"/"top"/"bottom"/"random") - crops image at selected vertical location
        :param h_pos: str("center"/"left"/"right"/"bottom") - crops image at selected horizontal location
        :param keep_size: keep_size: int(1/0) - resize cropped image to original size (default = 0)
        :return: Img class instance
        """
        self.image = crop.fixed_crop(self.image, height, width, v_pos, h_pos, keep_size)
        self.shape = self.image.shape
        return self

    def flip(self, axis = 0) -> 'Img':
        """
        Method flips the image around either vertically (over horizontal axis = 0),
        or horizontally (over vertical axis = 1).

        :param axis: int(0/1). Flip the image vertically (over horizontal axis = 0) or vertically (over vertical axis = 1)
        :return: Img class instance
        """
        self.image = flip.flip(self.image, axis)
        self.shape = self.image.shape
        return self

    def blur(self, a = 1) -> 'Img':
        """
        Method applies Gaussian blurring to the input, with the strength alpha. The method
        maintains the original size of the object, but inserts a black border.

        :param a: int(1 <= a < inf) - parameter to repeat the blurring a times. Strengthens the effect.
        :return: Img class instance
        """
        self.image = blur.blur(self.image, a)
        self.shape = self.image.shape
        return self

    def dropout(self, a = 0.5) -> 'Img':
        """
        Method drops out the selected fraction of pixels from the input image (replaces
        with black pixels).

        :param a: float(0 <= a <= 1) - the fraction of pixels to remove from the image (default = 0.5 = 50%)
        :return: Img class instance
        """
        self.image = dropout.dropous(self.image, a)
        self.shape = self.image.shape
        return self

    def channelshuffle(self) -> 'Img':
        """
        Method randomly shuffles image's channels (swaps them places).

        :return: Img class instance
        """
        self.image = color_trans.channelshuffle(self.image)
        self.shape = self.image.shape
        return self

    def colorjitter(self, channel, a, add_brightness=0) -> 'Img':
        """
        Method increases intensity of the selected channel by parameter alpha and optionally
        increases/decreases its brightness.

        :param channel: int(0 <= channel < 3) - selected channel to jitter (0 = R, 1 = G, 2 = B).
        :param a: float(-1 <= alpha <= 1) - the amount of intensity to apply.
        :param add_brightness: float(-1 <= add_brightness <= 1) - the amount of brightness to add (default = 0).
        :return: Img class instance
        """
        self.image = color_trans.colorjitter(self.image, channel, a, add_brightness)
        self.shape = self.image.shape
        return self

    def resize(self, a = 0, height = 0, width = 0) -> 'Img':
        """
        Method resizes image to either fixed height/width or to size equal to
        proportion of original size a.

        :param a: float(0 < a) - proportional resizing parameter alpha (returned dims = a * original dims)
        :param height: int(0 < height) - height of resized image (in px)
        :param width: int(0 < width) - width of resized image (in px)
        :return: Img class instance
        """
        self.image = resize.resize(self.image, a, height, width)
        self.shape = self.image.shape
        return self

    def combine(self, target_image, a = 0.5) -> 'Img':
        """
        Method combines the original image with a target image (rescaled to the same dims),
        using an opacity parameter a.

        :param target_image: Img() - an Img class instance type image, to combine with the first image.
        :param a: float(0 <= a <= 1) - opacity parameter (1 = original image only, 0 = target image only)
        :return: Img class instance
        """
        self.image = combine.combine(self.image, target_image.image, a)
        self.shape = self.image.shape
        return self
